@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.h,v 1.25 2003/12/09 22:23:05 marco Exp $
+ *  $Id: ephy-embed.h,v 1.28 2004/01/22 23:17:35 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_H
@@ -32,13 +32,13 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_EMBED			(ephy_embed_get_type ())
 #define EPHY_EMBED(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_EMBED, EphyEmbed))
-#define EPHY_EMBED_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED, EphyEmbedClass))
+#define EPHY_EMBED_IFACE(k)		(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED, EphyEmbedIFace))
 #define EPHY_IS_EMBED(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_EMBED))
-#define EPHY_IS_EMBED_CLASS(k)		(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED))
-#define EPHY_EMBED_GET_CLASS(inst)	(G_TYPE_INSTANCE_GET_INTERFACE ((inst), EPHY_TYPE_EMBED, EphyEmbedClass))
+#define EPHY_IS_EMBED_IFACE(k)		(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED))
+#define EPHY_EMBED_GET_IFACE(inst)	(G_TYPE_INSTANCE_GET_INTERFACE ((inst), EPHY_TYPE_EMBED, EphyEmbedIFace))
 
-typedef struct EphyEmbedClass	EphyEmbedClass;
-typedef struct _EphyEmbed	EphyEmbed;
+typedef struct EphyEmbed	EphyEmbed;
+typedef struct EphyEmbedIFace	EphyEmbedIFace;
 
 typedef enum
 {
@@ -75,9 +75,8 @@ typedef enum
 
 typedef enum
 {
-	EMBED_RELOAD_NORMAL	 = 1 << 1,
-	EMBED_RELOAD_BYPASSCACHE = 1 << 2,
-	EMBED_RELOAD_BYPASSPROXY = 1 << 3
+	EMBED_RELOAD_NORMAL	= 1 << 0,
+	EMBED_RELOAD_FORCE	= 1 << 1
 } EmbedReloadFlags;
 
 typedef struct
@@ -136,7 +135,7 @@ typedef enum
 	STATE_IS_SECURE_HIGH
 } EmbedSecurityLevel;
 
-struct EphyEmbedClass
+struct EphyEmbedIFace
 {
 	GTypeInterface base_iface;
 
@@ -228,6 +227,7 @@ struct EphyEmbedClass
 							 EmbedPrintPreviewNavType type,
 							 int page);
 	void		   (* activate)			(EphyEmbed *embed);
+	gboolean	   (* has_modified_forms)	(EphyEmbed *embed);
 };
 
 GType		  ephy_embed_get_type			(void);
@@ -319,6 +319,8 @@ void		  ephy_embed_print_preview_navigate	(EphyEmbed *embed,
 /* Misc. utility */
 void		  ephy_embed_activate			(EphyEmbed *embed);
 
+gboolean	  ephy_embed_has_modified_forms		(EphyEmbed *embed);
+
 G_END_DECLS
 
 #endif
