@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-encodings.c,v 1.2 2003/10/20 21:22:46 chpe Exp $
+ *  $Id: ephy-encodings.c,v 1.10 2004/01/22 21:00:38 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,7 +30,7 @@
 #include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <string.h>
 
 #define EPHY_ENCODINGS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_ENCODINGS, EphyEncodingsPrivate))
@@ -124,7 +124,6 @@ encoding_entries [] =
 	{ N_("Turkish (_MacTurkish)"),              "x-mac-turkish",         LG_TURKISH,	FALSE },
 	{ N_("Turkish (_Windows-1254)"),            "windows-1254",          LG_TURKISH,	FALSE },
 	{ N_("Unicode (UTF-_8)"),                   "UTF-8",                 LG_UNICODE,	FALSE },
-	{ N_("Unicode (UTF-_7)"),                   "UTF-7",                 LG_UNICODE,	FALSE },
 	{ N_("Cyrillic/Ukrainian (_KOI8-U)"),       "KOI8-U",                LG_UKRAINIAN,	FALSE },
 	{ N_("Cyrillic/Ukrainian (Mac_Ukrainian)"), "x-mac-ukrainian",       LG_UKRAINIAN,	FALSE },
 	{ N_("Vietnamese (_TCVN)"),                 "x-viet-tcvn5712",       LG_VIETNAMESE,	FALSE },
@@ -137,6 +136,15 @@ encoding_entries [] =
 	{ N_("Western (_MacRoman)"),                "x-mac-roman",           LG_WESTERN,	FALSE },
 	{ N_("Western (_Windows-1252)"),            "windows-1252",          LG_WESTERN,	FALSE },
 
+	/* the following encodings are so rarely used that we don't want to pollute the "related"
+	 * part of the encodings menu with them, so we set the language group to 0 here
+	 */
+	{ N_("English (_US-ASCII)"),                "us-ascii",              0,			FALSE },
+	{ N_("Unicode (UTF-_16 BE)"),               "UTF-16BE",              0,			FALSE },
+	{ N_("Unicode (UTF-1_6 LE)"),               "UTF-16LE",              0,			FALSE },
+	{ N_("Unicode (UTF-_32 BE)"),               "UTF-32BE",              0,			FALSE },
+	{ N_("Unicode (UTF-3_2 LE)"),               "UTF-32LE",              0,			FALSE },
+
 	{ N_("Off"),		 		    "",				   LG_NONE,								TRUE },
 	{ N_("Chinese"),			    "zh_parallel_state_machine",   LG_CHINESE_TRAD | LG_CHINESE_SIMP,					TRUE },
 	{ N_("Chinese Simplified"),		    "zhcn_parallel_state_machine", LG_CHINESE_SIMP,							TRUE },
@@ -226,13 +234,95 @@ ephy_encodings_class_init (EphyEncodingsClass *klass)
 	g_type_class_add_private (object_class, sizeof (EphyEncodingsPrivate));
 }
 
+static EphyNode *
+add_encoding (EphyEncodings *encodings,
+	      const char *title,
+	      const char *code,
+	      EphyLanguageGroup groups,
+	      gboolean is_autodetector)
+{
+	EphyNode *node;
+	char *elided, *normalised;
+	GValue value = { 0, };
+
+	node = ephy_node_new (encodings->priv->db);
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, title);
+	ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_TITLE, &value);
+	g_value_unset (&value);
+
+	elided = ephy_string_elide_underscores (title);
+	normalised = g_utf8_normalize (elided, -1, G_NORMALIZE_DEFAULT);
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_take_string (&value, g_utf8_collate_key (normalised, -1));
+	ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_COLLATION_KEY, &value);
+	g_value_unset (&value);
+
+	g_free (normalised);
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_take_string (&value, elided);
+	ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_TITLE_ELIDED, &value);
+	g_value_unset (&value);
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, code);
+	ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_ENCODING, &value);
+	g_value_unset (&value);
+
+	g_value_init (&value, G_TYPE_INT);
+	g_value_set_int (&value, groups);
+	ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_LANGUAGE_GROUPS, &value);
+	g_value_unset (&value);
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, is_autodetector);
+	ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_IS_AUTODETECTOR, &value);
+	g_value_unset (&value);
+
+	/* now insert the node in our structure */
+	ephy_node_add_child (encodings->priv->root, node);
+	g_hash_table_insert (encodings->priv->hash, g_strdup (code), node);
+
+	if (is_autodetector)
+	{
+		ephy_node_add_child (encodings->priv->detectors, node);
+	}
+	else
+	{
+		ephy_node_add_child (encodings->priv->encodings, node);
+	}
+
+	return node;
+}
+
 EphyNode *
 ephy_encodings_get_node (EphyEncodings *encodings,
-			 const char *code)
+			 const char *code,
+			 gboolean add_if_not_found)
 {
+	EphyNode *node;
+
 	g_return_val_if_fail (EPHY_IS_ENCODINGS (encodings), NULL);
 
-	return g_hash_table_lookup (encodings->priv->hash, code);
+	node = g_hash_table_lookup (encodings->priv->hash, code);
+
+	/* if it doesn't exist, add a node for it */
+	if (!EPHY_IS_NODE (node) && add_if_not_found)
+	{
+		char *title;
+
+		/* translators: this is the title that an unknown encoding will
+		 * be displayed as.
+		 */
+		title = g_strdup_printf (_("Unknown (%s)"), code);
+		node = add_encoding (encodings, title, code, 0, FALSE);
+		g_free (title);
+	}
+
+	return node;
 }
 
 GList *
@@ -259,31 +349,16 @@ ephy_encodings_get_encodings (EphyEncodings *encodings,
 			list = g_list_prepend (list, kid);
 		}
 	}
-	ephy_node_thaw (encodings->priv->encodings);
 
 	return list;
 }
 
-GList *
+EphyNode *
 ephy_encodings_get_detectors (EphyEncodings *encodings)
 {
-	GList *list = NULL;
-	GPtrArray *children;
-	int i, n_items;
+	g_return_val_if_fail (EPHY_IS_ENCODINGS (encodings), NULL);
 
-	children = ephy_node_get_children (encodings->priv->detectors);
-	n_items = children->len;
-	for (i = 0; i < n_items; i++)
-	{
-		EphyNode *kid;
-															     
-		kid = g_ptr_array_index (children, i);
-
-		list = g_list_prepend (list, kid);
-	}
-	ephy_node_thaw (encodings->priv->detectors);
-
-	return list;
+	return encodings->priv->detectors;
 }
 
 EphyNode *
@@ -302,13 +377,15 @@ ephy_encodings_add_recent (EphyEncodings *encodings,
 
 	g_return_if_fail (EPHY_IS_ENCODINGS (encodings));
 	g_return_if_fail (code != NULL);
-	g_return_if_fail (ephy_encodings_get_node (encodings, code) != NULL);
+	
+	if (ephy_encodings_get_node (encodings, code, FALSE) == NULL) return;
 
 	/* keep the list elements unique */
 	element = g_slist_find_custom (encodings->priv->recent, code,
 				       (GCompareFunc) strcmp);
 	if (element != NULL)
 	{
+		g_free (element->data);
 		encodings->priv->recent =
 			g_slist_remove_link (encodings->priv->recent, element);
 	}
@@ -342,7 +419,7 @@ ephy_encodings_get_recent (EphyEncodings *encodings)
 	{
 		EphyNode *node;
 
-		node = ephy_encodings_get_node (encodings, (char *) l->data);
+		node = ephy_encodings_get_node (encodings, (char *) l->data, FALSE);
 		g_return_val_if_fail (EPHY_IS_NODE (node), NULL);
 
 		list = g_list_prepend (list, node);
@@ -365,7 +442,9 @@ ephy_encodings_init (EphyEncodings *encodings)
 	db = ephy_node_db_new ("EncodingsDB");
 	encodings->priv->db = db;
 
-	encodings->priv->hash = g_hash_table_new (g_str_hash, g_str_equal);
+	encodings->priv->hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+						       (GDestroyNotify) g_free,
+						       NULL);
 
 	encodings->priv->root = ephy_node_new_with_id (db, ALL_NODE_ID);
 	encodings->priv->encodings = ephy_node_new_with_id (db, ENCODINGS_NODE_ID);
@@ -378,57 +457,11 @@ ephy_encodings_init (EphyEncodings *encodings)
 	/* now fill the db */
 	for (i = 0; i < n_encoding_entries; i++)
 	{
-		EphyNode *node;
-		char *elided, *normalised;
-		GValue value = { 0, };
-
-		node = ephy_node_new (db);
-		ephy_node_add_child (encodings->priv->root, node);
-		g_hash_table_insert (encodings->priv->hash, encoding_entries[i].code, node);
-	
-		if (encoding_entries[i].is_autodetector)
-		{
-			ephy_node_add_child (encodings->priv->detectors, node);
-		}
-		else
-		{
-			ephy_node_add_child (encodings->priv->encodings, node);
-		}
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, _(encoding_entries[i].title));
-		ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_TITLE, &value);
-		g_value_unset (&value);
-
-		elided = ephy_string_elide_underscores (_(encoding_entries[i].title));
-		normalised = g_utf8_normalize (elided, -1, G_NORMALIZE_DEFAULT);
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_take_string (&value, g_utf8_collate_key (normalised, -1));
-		ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_COLLATION_KEY, &value);
-		g_value_unset (&value);
-
-		g_free (normalised);
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_take_string (&value, elided);
-		ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_TITLE_ELIDED, &value);
-		g_value_unset (&value);
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, encoding_entries[i].code);
-		ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_ENCODING, &value);
-		g_value_unset (&value);
-
-		g_value_init (&value, G_TYPE_INT);
-		g_value_set_int (&value, encoding_entries[i].groups);
-		ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_LANGUAGE_GROUPS, &value);
-		g_value_unset (&value);
-
-		g_value_init (&value, G_TYPE_BOOLEAN);
-		g_value_set_boolean (&value, encoding_entries[i].is_autodetector);
-		ephy_node_set_property (node, EPHY_NODE_ENCODING_PROP_IS_AUTODETECTOR, &value);
-		g_value_unset (&value);
+		add_encoding (encodings,
+			      _(encoding_entries[i].title),
+			      encoding_entries[i].code,
+			      encoding_entries[i].groups,
+			      encoding_entries[i].is_autodetector);
 	}
 
 	/* get the list of recently used encodings */
@@ -442,7 +475,7 @@ ephy_encodings_init (EphyEncodings *encodings)
 	{
 		if (g_slist_find (encodings->priv->recent, l->data) == NULL
 		    && g_slist_length (encodings->priv->recent) < RECENT_MAX
-		    && ephy_encodings_get_node (encodings, l->data) != NULL)
+		    && ephy_encodings_get_node (encodings, l->data, FALSE) != NULL)
 		{
 			encodings->priv->recent =
 				g_slist_prepend (encodings->priv->recent,
@@ -472,6 +505,15 @@ ephy_encoding_info_free (EphyEncodingInfo *info)
 	}
 }
 
+gboolean
+ephy_encoding_info_is_automatic (EphyEncodingInfo *info)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+
+	return (info->encoding_source < EMBED_ENCODING_PARENT_FORCED)
+		&& (info->forced_encoding == NULL || info->forced_encoding[0] == '\0');
+}
+
 EphyEncodings *
 ephy_encodings_new (void)
 {
