@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Nate Case 
+ *  Copyright (C) 2000 Nate Case
  *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,15 +16,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-notifiers.cpp,v 1.39 2003/12/02 11:41:09 chpe Exp $
+ *  $Id: mozilla-notifiers.cpp,v 1.44 2004/01/09 09:29:54 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
+#include "ephy-embed-shell.h"
 #include "mozilla-notifiers.h"
 #include "eel-gconf-extensions.h"
 #include "MozRegisterComponents.h"
@@ -39,6 +39,8 @@
 #include <glib/gi18n.h>
 #include <stdlib.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtksettings.h>
+
 #include <nsCOMPtr.h>
 #include <nsIPrefService.h>
 #include <nsIServiceManager.h>
@@ -136,18 +138,20 @@ static const struct
 }
 conversion_table [] =
 {
-	{ CONF_SECURITY_JAVA_ENABLED, BOOL_PREF, "security.enable_java"},
-	{ CONF_SECURITY_JAVASCRIPT_ENABLED, BOOL_PREF, "javascript.enabled"},
-	{ CONF_NETWORK_PROXY_AUTO_URL, STRING_PREF, "network.proxy.autoconfig_url"},
-	{ CONF_NETWORK_HTTP_PROXY, STRING_PREF, "network.proxy.http"},
-	{ CONF_NETWORK_FTP_PROXY, STRING_PREF, "network.proxy.ftp"},
-	{ CONF_NETWORK_SSL_PROXY, STRING_PREF, "network.proxy.ssl"},
-	{ CONF_NETWORK_HTTP_PROXY_PORT, INT_PREF, "network.proxy.http_port"},
-	{ CONF_NETWORK_FTP_PROXY_PORT, INT_PREF, "network.proxy.ftp_port"},
-	{ CONF_NETWORK_SSL_PROXY_PORT, INT_PREF, "network.proxy.ssl_port"},
+	{ CONF_SECURITY_JAVA_ENABLED, BOOL_PREF, "security.enable_java" },
+	{ CONF_SECURITY_JAVASCRIPT_ENABLED, BOOL_PREF, "javascript.enabled" },
+	{ CONF_NETWORK_PROXY_AUTO_URL, STRING_PREF, "network.proxy.autoconfig_url" },
+	{ CONF_NETWORK_HTTP_PROXY, STRING_PREF, "network.proxy.http" },
+	{ CONF_NETWORK_FTP_PROXY, STRING_PREF, "network.proxy.ftp" },
+	{ CONF_NETWORK_SSL_PROXY, STRING_PREF, "network.proxy.ssl" },
+	{ CONF_NETWORK_SOCKS_PROXY, STRING_PREF, "network.proxy.socks" },
+	{ CONF_NETWORK_HTTP_PROXY_PORT, INT_PREF, "network.proxy.http_port" },
+	{ CONF_NETWORK_FTP_PROXY_PORT, INT_PREF, "network.proxy.ftp_port" },
+	{ CONF_NETWORK_SSL_PROXY_PORT, INT_PREF, "network.proxy.ssl_port" },
+	{ CONF_NETWORK_SOCKS_PROXY_PORT, INT_PREF, "network.proxy.socks_port" },
 	{ CONF_LANGUAGE_DEFAULT_ENCODING, STRING_PREF, "intl.charset.default" },
 	{ CONF_LANGUAGE_AUTODETECT_ENCODING, STRING_PREF, "intl.charset.detector" },
-	{ CONF_RENDERING_DEFAULT_FONT, STRING_PREF, "font.default" },
+	{ CONF_BROWSE_WITH_CARET, BOOL_PREF, "accessibility.browsewithcaret" },
 
 	{ NULL, 0, NULL }
 };
@@ -745,12 +749,14 @@ get_system_language ()
 		char *lang = (char *)sys_langs->data;
 
 		/* FIXME this probably need to be smarter */
+		/* FIXME this can be up to 8 chars, not just 2 */
 		if (strcmp (lang, "C") != 0)
 		{
 			return g_strndup (lang, 2);
 		}
 	}
 
+	/* fallback to english */
 	return g_strdup ("en");
 }
 
@@ -760,43 +766,68 @@ mozilla_language_notifier(GConfClient *client,
 			  GConfEntry *entry,
 			  EphyEmbedSingle *single)
 {
-	GSList *languages, *l;
+	GSList *languages, *l, *ulist = NULL;
 	GString *result;
 
 	languages = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
-	if (languages == NULL) return;
 
 	result = g_string_new ("");
 
+	/* substitute the system language */
+	l = g_slist_find_custom (languages, "system", (GCompareFunc) strcmp);
+	if (l != NULL)
+	{
+		char *sys_lang;
+		int index;
+
+		index = g_slist_position (languages, l);
+		g_free (l->data);
+		languages = g_slist_delete_link (languages, l);
+
+		sys_lang = get_system_language ();
+
+		if (sys_lang)
+		{
+			char **s;
+			int i = 0;
+
+			s = g_strsplit (sys_lang, ",", -1);
+			while (s[i] != NULL)
+			{
+				languages = g_slist_insert (languages, g_strdup (s[i]), index);
+
+				index ++;
+				i++;
+			}
+
+			g_strfreev (s);
+		}		
+	}
+
+	/* now make a list of unique entries */
 	for (l = languages; l != NULL; l = l->next)
 	{
-		char *lang = (char *)l->data;
+		if (g_slist_find_custom (ulist, l->data, (GCompareFunc) strcmp) == NULL)
+		{
+			ulist = g_slist_prepend (ulist, l->data);
+		}
+	}
+	ulist = g_slist_reverse (ulist);
 
-		if (strcmp (lang, "system") == 0)
-		{
-			char *sys_lang;
-			
-			sys_lang = get_system_language ();
-			if (sys_lang)
-			{
-				g_string_append (result, sys_lang);
-				g_free (sys_lang);
-			}
-		}
-		else
-		{
-			g_string_append (result, (char *)l->data);
-		}
+	for (l = ulist; l != NULL; l = l->next)
+	{
+		g_string_append (result, (char *)l->data);
 
 		if (l->next) g_string_append (result, ",");
 	}
-	
+
 	mozilla_prefs_set_string ("intl.accept_languages", result->str);
 
 	g_string_free (result, TRUE);
-	
+
 	g_slist_foreach (languages, (GFunc) g_free, NULL);
 	g_slist_free (languages);
+	g_slist_free (ulist);
 }
 
 static void
