@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-stock-icons.c,v 1.8 2003/11/22 14:40:36 marco Exp $
+ *  $Id: ephy-stock-icons.c,v 1.9 2003/12/13 22:04:02 chpe Exp $
  */
 
 #include <gtk/gtk.h>
@@ -40,7 +40,8 @@ ephy_stock_icons_init (void)
 		STOCK_FULLSCREEN,
 		STOCK_VIEW_SOURCE,
 		STOCK_SEND_MAIL,
-		STOCK_ADD_BOOKMARK
+		STOCK_ADD_BOOKMARK,
+		STOCK_PRINT_SETUP
 	};
 
 	static const char *items[] =
