@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-topic-action.c,v 1.32 2003/10/10 19:24:33 marco Exp $
+ *  $Id: ephy-topic-action.c,v 1.34 2003/12/16 17:29:43 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,7 +23,7 @@
 #endif
 
 #include <gtk/gtktoolitem.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 
 #include "ephy-topic-action.h"
 #include "ephy-node-common.h"
@@ -293,8 +293,6 @@ append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node,
 
 		g_list_free (node_list);
 	}
-
-	ephy_node_thaw (node);
 }
 
 static GtkWidget *
@@ -400,7 +398,6 @@ build_topics_menu (EphyTopicAction *action, EphyNode *node)
 		gtk_widget_show (bmk_menu);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), bmk_menu);
 	}
-	ephy_node_thaw (node);
 	g_list_free (node_list);
 
 	uncategorized = ephy_bookmarks_get_not_categorized (bookmarks);
