@@ -156,6 +156,8 @@ main (int argc, char *argv[])
 	ephy_stock_icons_init ();
 	eel_gconf_monitor_add ("/apps/epiphany/general");
 	eel_gconf_monitor_add ("/apps/epiphany/lockdown");
+	eel_gconf_monitor_add ("/desktop/gnome/lockdown");
+
 	bonobo_activate ();
 
 	ephy_shell = ephy_shell_new ();
@@ -182,6 +184,7 @@ main (int argc, char *argv[])
 
 	eel_gconf_monitor_remove ("/apps/epiphany/general");
 	eel_gconf_monitor_remove ("/apps/epiphany/lockdown");
+	eel_gconf_monitor_remove ("/desktop/gnome/lockdown");
 	ephy_state_save ();
 	ephy_file_helpers_shutdown ();
 	gnome_vfs_shutdown ();
