@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.h,v 1.24 2003/12/07 10:23:31 chpe Exp $
+ *  $Id: ephy-shell.h,v 1.25 2004/01/22 19:23:06 clahey Exp $
  */
 
 #ifndef EPHY_SHELL_H
@@ -136,9 +136,6 @@ GObject        *ephy_shell_get_prefs_dialog		(EphyShell *shell);
 
 GObject	       *ephy_shell_get_print_setup_dialog	(EphyShell *shell);
 
-void		ephy_shell_delete_on_exit		(EphyShell *shell,
-							 const char *path);
-
 G_END_DECLS
 
 #endif
