@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.80 2004/01/12 19:33:03 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.84 2004/01/23 22:50:02 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -412,6 +412,8 @@ address_has_web_scheme (const char *address)
 				  strcmp (scheme, "https") == 0 ||
 				  strcmp (scheme, "ftp") == 0 ||
 				  strcmp (scheme, "file") == 0 ||
+				  strcmp (scheme, "data") == 0 ||
+				  strcmp (scheme, "about") == 0 ||
 				  strcmp (scheme, "gopher") == 0);
 
 		gnome_vfs_uri_unref (uri);
@@ -1062,7 +1064,7 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	/* shift+click saves the link target */
 	else if (is_link && is_left_click && with_shift)
 	{
-		save_property_url (embed, event, "link", CONF_STATE_DOWNLOAD_DIR);
+		save_property_url (embed, event, "link", CONF_STATE_SAVE_DIR);
 	}
 	/* shift+click saves the non-link image
 	 * Note: pressing enter to submit a form synthesizes a mouse click event
@@ -1146,48 +1148,48 @@ ephy_tab_init (EphyTab *tab)
 	/* set a pointer in the embed's widget back to the tab */
 	g_object_set_data (embed_widget, "ephy-tab", tab);
 
-	g_signal_connect (embed_widget, "parent_set",
-			  G_CALLBACK (ephy_tab_parent_set_cb),
-			  tab);
-	g_signal_connect (embed_widget, "destroy",
-			  G_CALLBACK (ephy_tab_embed_destroy_cb),
-			  tab);
-	g_signal_connect (embed, "link_message",
-			  G_CALLBACK (ephy_tab_link_message_cb),
-			  tab);
-	g_signal_connect (embed, "ge_location",
-			  G_CALLBACK (ephy_tab_address_cb),
-			  tab);
-	g_signal_connect (embed, "title",
-			  G_CALLBACK (ephy_tab_title_cb),
-			  tab);
-	g_signal_connect (embed, "ge_zoom_change",
-			  G_CALLBACK (ephy_tab_zoom_changed_cb),
-			  tab);
-	g_signal_connect (embed, "ge_net_state",
-			  G_CALLBACK (ephy_tab_net_state_cb),
-			  tab);
-	g_signal_connect (embed, "ge_new_window",
-			  G_CALLBACK (ephy_tab_new_window_cb),
-			  tab);
-	g_signal_connect (embed, "visibility",
-			  G_CALLBACK (ephy_tab_visibility_cb),
-			  tab);
-	g_signal_connect (embed, "destroy_browser",
-			  G_CALLBACK (ephy_tab_destroy_brsr_cb),
-			  tab);
-	g_signal_connect (embed, "size_to",
-			  G_CALLBACK (ephy_tab_size_to_cb),
-			  tab);
-	g_signal_connect (embed, "ge_dom_mouse_click",
-			  G_CALLBACK (ephy_tab_dom_mouse_click_cb),
-			  tab);
-	g_signal_connect (embed, "ge_security_change",
-			  G_CALLBACK (ephy_tab_security_change_cb),
-			  tab);
-	g_signal_connect (embed, "ge_favicon",
-			  G_CALLBACK (ephy_tab_favicon_cb),
-			  tab);
+	g_signal_connect_object (embed_widget, "parent_set",
+				 G_CALLBACK (ephy_tab_parent_set_cb),
+				 tab, 0);
+	g_signal_connect_object (embed_widget, "destroy",
+				 G_CALLBACK (ephy_tab_embed_destroy_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "link_message",
+				 G_CALLBACK (ephy_tab_link_message_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_location",
+				 G_CALLBACK (ephy_tab_address_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "title",
+				 G_CALLBACK (ephy_tab_title_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_zoom_change",
+				 G_CALLBACK (ephy_tab_zoom_changed_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_net_state",
+				 G_CALLBACK (ephy_tab_net_state_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_new_window",
+				 G_CALLBACK (ephy_tab_new_window_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "visibility",
+				 G_CALLBACK (ephy_tab_visibility_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "destroy_browser",
+				 G_CALLBACK (ephy_tab_destroy_brsr_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "size_to",
+				 G_CALLBACK (ephy_tab_size_to_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_dom_mouse_click",
+				 G_CALLBACK (ephy_tab_dom_mouse_click_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_security_change",
+				 G_CALLBACK (ephy_tab_security_change_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_favicon",
+				 G_CALLBACK (ephy_tab_favicon_cb),
+				 tab, 0);
 
 	cache = EPHY_FAVICON_CACHE
 		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
