@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.149 2003/12/25 14:48:12 chpe Exp $
+ *  $Id: ephy-window.c,v 1.165 2004/01/21 23:39:43 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,7 @@
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-embed-prefs.h"
 #include "ephy-zoom.h"
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
@@ -57,6 +58,8 @@
 #include <gtk/gtktoggleaction.h>
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtktoggleaction.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtkmessagedialog.h>
 
 static void ephy_window_class_init		(EphyWindowClass *klass);
 static void ephy_window_init			(EphyWindow *gs);
@@ -238,7 +241,10 @@ static GtkToggleActionEntry ephy_menu_toggle_entries [] =
 	  G_CALLBACK (window_cmd_view_statusbar), TRUE },
 	{ "ViewFullscreen", STOCK_FULLSCREEN, N_("_Fullscreen"), "F11",
 	  N_("Browse at full screen"),
-	  G_CALLBACK (window_cmd_view_fullscreen), FALSE}
+	  G_CALLBACK (window_cmd_view_fullscreen), FALSE },
+	{ "BrowseWithCaret", NULL, N_("Selection Caret"), "F7",
+	  "",
+	  G_CALLBACK (window_cmd_browse_with_caret), FALSE }
 };
 static guint ephy_menu_n_toggle_entries = G_N_ELEMENTS (ephy_menu_toggle_entries);
 
@@ -265,8 +271,10 @@ static GtkActionEntry ephy_popups_entries [] = {
 	  NULL, G_CALLBACK (popup_cmd_link_in_new_window) },
 	{ "OpenLinkInNewTab", NULL, N_("Open Link in New _Tab"),
 	  NULL, NULL, G_CALLBACK (popup_cmd_link_in_new_tab) },
-	{ "DownloadLink", GTK_STOCK_SAVE, N_("_Download Link"), NULL,
+	{ "DownloadLink", NULL, N_("_Download Link"), NULL,
 	  NULL, G_CALLBACK (popup_cmd_download_link) },
+	{ "DownloadLinkAs", GTK_STOCK_SAVE_AS, N_("_Save Link As..."), NULL,
+	  NULL, G_CALLBACK (popup_cmd_download_link_as) },
 	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("_Bookmark Link..."),
 	  NULL, NULL, G_CALLBACK (popup_cmd_bookmark_link) },
 	{ "CopyLinkAddress", NULL, N_("_Copy Link Address"), NULL,
@@ -290,6 +298,7 @@ static guint ephy_popups_n_entries = G_N_ELEMENTS (ephy_popups_entries);
 
 #define CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME  "/apps/epiphany/lockdown/disable_javascript_chrome"
 #define CONF_LOCKDOWN_HIDE_MENUBAR "/apps/epiphany/lockdown/hide_menubar"
+#define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
 
 #define EPHY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_WINDOW, EphyWindowPrivate))
 
@@ -319,12 +328,22 @@ struct EphyWindowPrivate
 	gboolean has_size;
 	guint num_tabs;
 	guint tab_message_cid;
+	guint help_message_cid;
 
 	guint disable_js_chrome_notifier_id;
 	guint show_toolbars_notifier_id;
 	guint show_bookmarks_bar_notifier_id;
 	guint show_statusbar_notifier_id;
 	guint hide_menubar_notifier_id;
+	guint disable_arbitrary_url_notifier_id;
+	guint disable_bookmark_editing_notifier_id;
+	guint disable_toolbar_editing_notifier_id;
+	guint disable_history_notifier_id;
+	guint disable_printing_notifier_id;
+	guint disable_print_setup_notifier_id;
+	guint disable_save_to_disk_notifier_id;
+	guint disable_command_line_notifier_id;
+	guint browse_with_caret_notifier_id;
 };
 
 enum
@@ -417,6 +436,8 @@ update_exit_fullscreen_popup_position (EphyWindow *window)
 	GdkRectangle screen_rect;
 	int popup_height;
 
+	g_return_if_fail (window->priv->exit_fullscreen_popup != NULL);
+
 	gtk_window_get_size (GTK_WINDOW (window->priv->exit_fullscreen_popup),
                              NULL, &popup_height);
 
@@ -584,7 +605,7 @@ ephy_window_fullscreen (EphyWindow *window)
 
 	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
                           "size-changed", G_CALLBACK (size_changed_cb),
-			  popup);
+			  window);
 
 	update_chromes_visibility (window);
 }
@@ -636,9 +657,108 @@ ephy_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, EphyW
 	return FALSE;
 }
 
+static gboolean
+confirm_close_with_modified_forms (EphyWindow *window)
+{
+	GtkWidget *dialog;
+	GtkWidget *hbox, *vbox, *label, *image;
+	char *text;
+	int response;
+
+	dialog = gtk_dialog_new_with_buttons ("",
+					      GTK_WINDOW (window),
+					      GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_MODAL,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      _("_Close document"), GTK_RESPONSE_OK,
+					      NULL);
+	
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
+
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
+			    TRUE, TRUE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
+					  GTK_ICON_SIZE_DIALOG);
+	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+
+	label = gtk_label_new (NULL);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+
+	text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+				_("There are unsubmitted changes to form elements."),
+				_("If you close the document anyway, you will lose that information."));
+	gtk_label_set_markup (GTK_LABEL (label), text);
+	g_free (text);
+
+	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+	gtk_widget_show (label);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	return response == GTK_RESPONSE_OK;
+}
+
+static gboolean
+ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *window)
+{
+	EphyTab *modified_tab = NULL;
+	GList *tabs, *l;
+	gboolean modified = FALSE;
+
+	tabs = ephy_window_get_tabs (window);
+	for (l = tabs; l != NULL; l = l->next)
+	{
+		EphyTab *tab = (EphyTab *) l->data;
+		EphyEmbed *embed;
+
+		g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
+
+		embed = ephy_tab_get_embed (tab);
+		g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
+
+		if (ephy_embed_has_modified_forms (embed))
+		{
+			modified = TRUE;
+			modified_tab = tab;
+			break;
+		}
+	}
+	g_list_free (tabs);
+
+	if (modified)
+	{
+		/* jump to the first tab with modified forms */
+		ephy_window_jump_to_tab (window, modified_tab);
+
+		if (confirm_close_with_modified_forms (window) == FALSE)
+		{
+			/* stop window close */
+			return TRUE;
+		}
+	}
+	
+	/* See bug #114689 */
+	gtk_widget_hide (widget);
+
+	/* proceed with window close */
+	return FALSE;
+}
 
 static void
-update_edit_actions_sensitivity (EphyWindow *window)
+update_edit_actions_sensitivity (EphyWindow *window, gboolean hide)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 	GtkActionGroup *action_group;
@@ -680,27 +800,19 @@ update_edit_actions_sensitivity (EphyWindow *window)
 	action_group = window->priv->action_group;
 
 	action = gtk_action_group_get_action (action_group, "EditCopy");
-	g_object_set (action, "sensitive", can_copy, NULL);
+	g_object_set (action, "sensitive", can_copy, "visible", !hide || can_copy, NULL);
 	action = gtk_action_group_get_action (action_group, "EditCut");
-	g_object_set (action, "sensitive", can_cut, NULL);
+	g_object_set (action, "sensitive", can_cut, "visible", !hide || can_cut, NULL);
 	action = gtk_action_group_get_action (action_group, "EditPaste");
-	g_object_set (action, "sensitive", can_paste, NULL);
+	g_object_set (action, "sensitive", can_paste, "visible", !hide || can_paste, NULL);
 	action = gtk_action_group_get_action (action_group, "EditUndo");
-	g_object_set (action, "sensitive", can_undo, NULL);
+	g_object_set (action, "sensitive", can_undo, "visible", !hide || can_undo, NULL);
 	action = gtk_action_group_get_action (action_group, "EditRedo");
-	g_object_set (action, "sensitive", can_redo, NULL);
+	g_object_set (action, "sensitive", can_redo, "visible", !hide || can_redo, NULL);
 }
 
 static void
-edit_menu_show_cb (GtkWidget *menu,
-		   EphyWindow *window)
-{
-	update_edit_actions_sensitivity (window);
-}
-
-static void
-edit_menu_hide_cb (GtkWidget *menu,
-		   EphyWindow *window)
+enable_edit_actions_sensitivity (EphyWindow *window)
 {
 	GtkActionGroup *action_group;
 	GtkAction *action;
@@ -708,13 +820,29 @@ edit_menu_hide_cb (GtkWidget *menu,
 	action_group = window->priv->action_group;
 
 	action = gtk_action_group_get_action (action_group, "EditCopy");
-	g_object_set (action, "sensitive", TRUE, NULL);
+	g_object_set (action, "sensitive", TRUE, "visible", TRUE, NULL);
 	action = gtk_action_group_get_action (action_group, "EditCut");
-	g_object_set (action, "sensitive", TRUE, NULL);
+	g_object_set (action, "sensitive", TRUE, "visible", TRUE, NULL);
+	action = gtk_action_group_get_action (action_group, "EditPaste");
+	g_object_set (action, "sensitive", TRUE, "visible", TRUE, NULL);
 	action = gtk_action_group_get_action (action_group, "EditUndo");
-	g_object_set (action, "sensitive", TRUE, NULL);
+	g_object_set (action, "sensitive", TRUE, "visible", TRUE, NULL);
 	action = gtk_action_group_get_action (action_group, "EditRedo");
-	g_object_set (action, "sensitive", TRUE, NULL);
+	g_object_set (action, "sensitive", TRUE, "visible", TRUE, NULL);
+}
+
+static void
+edit_menu_show_cb (GtkWidget *menu,
+		   EphyWindow *window)
+{
+	update_edit_actions_sensitivity (window, FALSE);
+}
+
+static void
+edit_menu_hide_cb (GtkWidget *menu,
+		   EphyWindow *window)
+{
+	enable_edit_actions_sensitivity (window);
 }
 
 static void
@@ -732,6 +860,63 @@ init_menu_updaters (EphyWindow *window)
 			  G_CALLBACK (edit_menu_hide_cb), window);
 }
 
+static void
+menu_item_select_cb (GtkMenuItem *proxy,
+		     EphyWindow *window)
+{
+	GtkAction *action;
+	char *message;
+
+	action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+	g_return_if_fail (action != NULL);
+	
+	g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+	if (message)
+	{
+		gtk_statusbar_push (GTK_STATUSBAR (window->priv->statusbar),
+				    window->priv->help_message_cid, message);
+		g_free (message);
+	}
+}
+
+static void
+menu_item_deselect_cb (GtkMenuItem *proxy,
+		       EphyWindow *window)
+{
+	gtk_statusbar_pop (GTK_STATUSBAR (window->priv->statusbar),
+			   window->priv->help_message_cid);
+}
+
+static void
+disconnect_proxy_cb (GtkUIManager *manager,
+		     GtkAction *action,
+		     GtkWidget *proxy,
+		     EphyWindow *window)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_handlers_disconnect_by_func
+			(proxy, G_CALLBACK (menu_item_select_cb), window);
+		g_signal_handlers_disconnect_by_func
+			(proxy, G_CALLBACK (menu_item_deselect_cb), window);
+	}
+}
+
+static void
+connect_proxy_cb (GtkUIManager *manager,
+		  GtkAction *action,
+		  GtkWidget *proxy,
+		  EphyWindow *window)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_connect (proxy, "select",
+				  G_CALLBACK (menu_item_select_cb), window);
+		g_signal_connect (proxy, "deselect",
+				  G_CALLBACK (menu_item_deselect_cb), window);
+	}
+}
+
 static void
 setup_ui_manager (EphyWindow *window)
 {
@@ -753,6 +938,11 @@ setup_ui_manager (EphyWindow *window)
 
 	merge = gtk_ui_manager_new ();
 
+	g_signal_connect (merge, "connect_proxy",
+			  G_CALLBACK (connect_proxy_cb), window);
+	g_signal_connect (merge, "disconnect_proxy",
+			  G_CALLBACK (disconnect_proxy_cb), window);
+
 	action_group = gtk_action_group_new ("WindowActions");
 	gtk_action_group_set_translation_domain (action_group, NULL);
 	gtk_action_group_add_actions (action_group, ephy_menu_entries,
@@ -815,7 +1005,6 @@ sync_tab_address (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	if (window->priv->closing) return;
 
 	address = ephy_tab_get_location (tab);
-
 	if (address == NULL)
 	{
 		address = "";
@@ -888,6 +1077,7 @@ sync_tab_navigation (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	GtkActionGroup *action_group;
 	GtkAction *action;
 	gboolean up = FALSE, back = FALSE, forward = FALSE;
+	gboolean disable_arbitrary_url, disable_history;
 
 	if (window->priv->closing) return;
 
@@ -906,6 +1096,14 @@ sync_tab_navigation (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 		forward = TRUE;
 	}
 
+	disable_arbitrary_url = eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL);
+	disable_history = eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_HISTORY);
+
+	up = up && !disable_arbitrary_url;
+
+	back = back && !disable_history;
+	forward = forward && !disable_history;
+
 	action_group = window->priv->action_group;
 	action = gtk_action_group_get_action (action_group, "GoUp");
 	g_object_set (action, "sensitive", up, NULL);
@@ -1099,13 +1297,20 @@ static void
 popup_menu_at_coords (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 		      gpointer user_data)
 {
-	EphyEmbedEvent *event = user_data;
+	EphyEmbedEvent *event = (EphyEmbedEvent *) user_data;
+
+	ephy_embed_event_get_coords (event, x, y);
 
-	*x = event->x;
-	*y = event->y;
 	*push_in = TRUE;
 }
 
+static void
+hide_embed_popup_cb (GtkWidget *popup,
+			EphyWindow *window)
+{
+	enable_edit_actions_sensitivity (window);
+}
+
 static void
 show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 {
@@ -1114,9 +1319,10 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	EmbedEventContext context;
 	const char *popup;
 	const GValue *value;
-	gboolean framed, has_background;
+	gboolean framed, has_background, can_open_in_new;
 	GtkWidget *widget;
 	EphyEmbedEventType type;
+	gboolean showing_edit_actions = FALSE;
 
 	/* Do not show the menu in print preview mode */
 	if (window->priv->is_ppview)
@@ -1128,6 +1334,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	framed = g_value_get_int (value);
 
 	has_background = ephy_embed_event_has_property (event, "background_image");
+	can_open_in_new = ephy_embed_event_has_property (event, "link-has-web-scheme");
 
 	context = ephy_embed_event_get_context (event);
 
@@ -1138,6 +1345,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	}
 	else if (context & EMBED_CONTEXT_LINK)
 	{
+		showing_edit_actions = TRUE;
 		popup = "/EphyLinkPopup";
 	}
 	else if (context & EMBED_CONTEXT_IMAGE)
@@ -1147,29 +1355,42 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 #if MOZILLA_SNAPSHOT > 12
 	else if (context & EMBED_CONTEXT_INPUT)
 	{
-		update_edit_actions_sensitivity (window);
+		showing_edit_actions = TRUE;
 		popup = "/EphyInputPopup";
 	}
 #endif
 	else
 	{
+		showing_edit_actions = TRUE;
 		popup = framed ? "/EphyFramedDocumentPopup" :
 				 "/EphyDocumentPopup";
 	}
 
-	action_group = window->priv->popups_action_group;
-	action = gtk_action_group_get_action (action_group, "SaveBackgroundAs");
-	g_object_set (action, "sensitive", has_background,
-			      "visible", has_background, NULL);
-
 	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER (window->ui_merge),
 				            popup);
 	g_return_if_fail (widget != NULL);
 
+	action_group = window->priv->popups_action_group;
+	action = gtk_action_group_get_action (action_group, "SaveBackgroundAs");
+	g_object_set (action, "sensitive", has_background,
+			      "visible", has_background, NULL);
+	action = gtk_action_group_get_action (action_group, "OpenLinkInNewWindow");
+	g_object_set (action, "sensitive", can_open_in_new, FALSE);
+	action = gtk_action_group_get_action (action_group, "OpenLinkInNewTab");
+	g_object_set (action, "sensitive", can_open_in_new, FALSE);
+
+	if (showing_edit_actions)
+	{
+		update_edit_actions_sensitivity (window, TRUE);
+	}
+
 	g_object_set_data_full (G_OBJECT (window), "context_event",
 				g_object_ref (event),
 				(GDestroyNotify)g_object_unref);
 
+	g_signal_connect (widget, "hide",
+			  G_CALLBACK (hide_embed_popup_cb), window);
+
 	type = ephy_embed_event_get_event_type (event);
 	if (type == EPHY_EMBED_EVENT_KEY)
 	{
@@ -1196,7 +1417,7 @@ tab_context_menu_cb (EphyEmbed *embed,
 	g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
 	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
 
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "EphyTab"));
+	tab = ephy_tab_for_embed (embed);
 	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
 	g_return_val_if_fail (window->priv->active_tab == tab, FALSE);
 
@@ -1356,12 +1577,12 @@ update_tabs_menu_sensitivity (EphyWindow *window)
 }
 
 static void
-tab_added_cb (EphyNotebook *notebook, GtkWidget *child, EphyWindow *window)
+tab_added_cb (EphyNotebook *notebook, EphyEmbed *embed, EphyWindow *window)
 {
 	EphyTab *tab;
 
-	g_return_if_fail (EPHY_IS_EMBED (child));
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
+        g_return_if_fail (EPHY_IS_EMBED (embed));
+	tab = ephy_tab_for_embed (embed);
 
 	window->priv->num_tabs++;
 
@@ -1372,12 +1593,12 @@ tab_added_cb (EphyNotebook *notebook, GtkWidget *child, EphyWindow *window)
 }
 
 static void
-tab_removed_cb (EphyNotebook *notebook, GtkWidget *child, EphyWindow *window)
+tab_removed_cb (EphyNotebook *notebook, EphyEmbed *embed, EphyWindow *window)
 {
 	EphyTab *tab;
 
-	g_return_if_fail (EPHY_IS_EMBED (child));
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
+        g_return_if_fail (EPHY_IS_EMBED (embed));
+	tab = ephy_tab_for_embed (embed);
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (tab),
 					      G_CALLBACK (sync_tab_visibility),
@@ -1419,6 +1640,19 @@ tabs_reordered_cb (EphyNotebook *notebook, EphyWindow *window)
 	ephy_tabs_menu_update (window->priv->tabs_menu);
 }
 
+static gboolean
+tab_delete_cb (EphyNotebook *notebook, GtkWidget *child, EphyWindow *window)
+{
+	g_return_val_if_fail (EPHY_IS_EMBED (child), FALSE);
+
+	if (ephy_embed_has_modified_forms (EPHY_EMBED (child)))
+	{
+		return !confirm_close_with_modified_forms (window);
+	}
+
+	return FALSE;
+}
+
 static GtkNotebook *
 setup_notebook (EphyWindow *window)
 {
@@ -1442,6 +1676,8 @@ setup_notebook (EphyWindow *window)
 			  G_CALLBACK (tab_detached_cb), NULL);
 	g_signal_connect (G_OBJECT (notebook), "tabs_reordered",
 			  G_CALLBACK (tabs_reordered_cb), window);
+	g_signal_connect (G_OBJECT (notebook), "tab_delete",
+			  G_CALLBACK (tab_delete_cb), window);
 
 	return notebook;
 }
@@ -1541,22 +1777,79 @@ ensure_default_icon (void)
 }
 
 static void
-update_layout_toggles (EphyWindow *window)
+update_actions (EphyWindow *window)
 {
 	GtkActionGroup *action_group = GTK_ACTION_GROUP (window->priv->action_group);
+	GtkActionGroup *popups_action_group = GTK_ACTION_GROUP (window->priv->popups_action_group);
 	GtkAction *action;
+	gboolean bookmarks_editable, save_to_disk;
+	gboolean printing, print_setup;
 
 	action = gtk_action_group_get_action (action_group, "ViewToolbar");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 				      eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS));
+	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_WINDOWS_SHOW_TOOLBARS), NULL);
 
 	action = gtk_action_group_get_action (action_group, "ViewBookmarksBar");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 				      eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR));
+	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_WINDOWS_SHOW_BOOKMARKS_BAR), NULL);
 
 	action = gtk_action_group_get_action (action_group, "ViewStatusbar");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 				      eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR));
+	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_WINDOWS_SHOW_STATUSBAR), NULL);
+
+	action = gtk_action_group_get_action (action_group, "GoLocation");
+	g_object_set (action, "sensitive", ! eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL), NULL);
+
+	action = gtk_action_group_get_action (action_group, "GoHistory");
+	g_object_set (action, "sensitive", ! eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_HISTORY), NULL);
+
+	bookmarks_editable = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING);
+	action = gtk_action_group_get_action (action_group, "GoBookmarks");
+	g_object_set (action, "sensitive", bookmarks_editable, NULL);
+	action = gtk_action_group_get_action (action_group, "FileBookmarkPage");
+	g_object_set (action, "sensitive", bookmarks_editable, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "ContextBookmarkPage");
+	g_object_set (action, "sensitive", bookmarks_editable, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "BookmarkLink");
+	g_object_set (action, "sensitive", bookmarks_editable, NULL);
+
+	save_to_disk = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_SAVE_TO_DISK);
+	action = gtk_action_group_get_action (action_group, "FileSaveAs");
+	g_object_set (action, "sensitive", save_to_disk, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "DownloadLink");
+	g_object_set (action, "sensitive", save_to_disk, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "SaveBackgroundAs");
+	g_object_set (action, "sensitive", save_to_disk, NULL);
+	action = gtk_action_group_get_action (popups_action_group, "SaveImageAs");
+	g_object_set (action, "sensitive", save_to_disk, NULL);
+
+	printing = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINTING);
+	print_setup = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) &&
+		!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE);
+	action = gtk_action_group_get_action (action_group, "FilePrintSetup");
+	g_object_set (action, "sensitive", printing && print_setup, NULL);
+	action = gtk_action_group_get_action (action_group, "FilePrintPreview");
+	g_object_set (action, "sensitive", printing && print_setup, NULL);
+	action = gtk_action_group_get_action (action_group, "FilePrint");
+	g_object_set (action, "sensitive", printing, NULL);
+
+	action = gtk_action_group_get_action (popups_action_group, "SetImageAsBackground");
+	g_object_set (action, "sensitive", eel_gconf_key_is_writable (CONF_DESKTOP_BG_PICTURE), NULL);
+
+	action = gtk_action_group_get_action (action_group, "EditToolbar");
+	g_object_set (action, "sensitive", ! eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_TOOLBAR_EDITING), NULL);
+}
+
+static void
+update_navigation (EphyWindow *window)
+{
+	if (window->priv->active_tab)
+	{
+		sync_tab_navigation (window->priv->active_tab, NULL, window);
+	}
 }
 
 static void
@@ -1566,7 +1859,40 @@ chrome_notifier (GConfClient *client,
 		 EphyWindow *window)
 {
 	update_chrome (window);
-	update_layout_toggles(window);
+	update_actions(window);
+}
+
+static void
+actions_notifier (GConfClient *client,
+		  guint cnxn_id,
+		  GConfEntry *entry,
+		  EphyWindow *window)
+{
+	update_actions(window);
+}
+
+static void
+navigation_notifier (GConfClient *client,
+		   guint cnxn_id,
+		   GConfEntry *entry,
+		   EphyWindow *window)
+{
+	update_navigation(window);
+	update_actions(window);
+}
+
+static void
+browse_with_caret_notifier (GConfClient *client,
+			    guint cnxn_id,
+			    GConfEntry *entry,
+			    EphyWindow *window)
+{
+	GtkAction *action;
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "BrowseWithCaret");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      eel_gconf_get_boolean (CONF_BROWSE_WITH_CARET));
 }
 
 static void
@@ -1614,6 +1940,9 @@ ephy_window_init (EphyWindow *window)
 	window->priv->tab_message_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR (window->priv->statusbar), "tab_message");
 
+	window->priv->help_message_cid = gtk_statusbar_get_context_id
+		(GTK_STATUSBAR (window->priv->statusbar), "help_message");
+
 	/* Initialize the menus */
 	window->priv->tabs_menu = ephy_tabs_menu_new (window);
 	window->priv->fav_menu = ephy_favorites_menu_new (window);
@@ -1636,6 +1965,9 @@ ephy_window_init (EphyWindow *window)
 	g_signal_connect (window, "window-state-event",
 			  G_CALLBACK (ephy_window_state_event_cb),
 			  window);
+	g_signal_connect (window, "delete-event",
+			  G_CALLBACK (ephy_window_delete_event_cb),
+			  window);
 
 	/* lockdown pref notifiers */
 	window->priv->disable_js_chrome_notifier_id = eel_gconf_notification_add
@@ -1658,6 +1990,44 @@ ephy_window_init (EphyWindow *window)
 		(CONF_LOCKDOWN_HIDE_MENUBAR,
 		 (GConfClientNotifyFunc)chrome_notifier, window);
 
+	window->priv->disable_arbitrary_url_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_ARBITRARY_URL,
+		 (GConfClientNotifyFunc)navigation_notifier, window);
+
+	window->priv->disable_bookmark_editing_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING,
+		 (GConfClientNotifyFunc)actions_notifier, window);
+
+	window->priv->disable_toolbar_editing_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_TOOLBAR_EDITING,
+		 (GConfClientNotifyFunc)actions_notifier, window);
+
+	window->priv->disable_history_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_HISTORY,
+		 (GConfClientNotifyFunc)navigation_notifier, window);
+
+	window->priv->disable_printing_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_PRINTING,
+		 (GConfClientNotifyFunc)actions_notifier, window);
+
+	window->priv->disable_print_setup_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_PRINT_SETUP,
+		 (GConfClientNotifyFunc)actions_notifier, window);
+
+	window->priv->disable_save_to_disk_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_SAVE_TO_DISK,
+		 (GConfClientNotifyFunc)actions_notifier, window);
+
+	window->priv->disable_command_line_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_COMMAND_LINE,
+		 (GConfClientNotifyFunc)actions_notifier, window);
+
+	/* other notifiers */
+	browse_with_caret_notifier (NULL, 0, NULL, window);
+	window->priv->browse_with_caret_notifier_id = eel_gconf_notification_add
+		(CONF_BROWSE_WITH_CARET,
+		 (GConfClientNotifyFunc)browse_with_caret_notifier, window);
+
 	/* ensure the UI is updated */
 	gtk_ui_manager_ensure_update (GTK_UI_MANAGER (window->ui_merge));
 
@@ -1674,6 +2044,15 @@ ephy_window_finalize (GObject *object)
 	eel_gconf_notification_remove (window->priv->show_bookmarks_bar_notifier_id);
 	eel_gconf_notification_remove (window->priv->show_statusbar_notifier_id);
 	eel_gconf_notification_remove (window->priv->hide_menubar_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_arbitrary_url_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_bookmark_editing_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_toolbar_editing_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_history_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_printing_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_print_setup_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_save_to_disk_notifier_id);
+	eel_gconf_notification_remove (window->priv->disable_command_line_notifier_id);
+	eel_gconf_notification_remove (window->priv->browse_with_caret_notifier_id);
 
 	if (window->priv->find_dialog)
 	{
@@ -1798,7 +2177,7 @@ real_get_active_tab (EphyWindow *window, int page_num)
 						  page_num);
 
 	g_return_val_if_fail (GTK_IS_WIDGET (embed_widget), NULL);
-	tab = g_object_get_data (G_OBJECT (embed_widget), "EphyTab");
+	tab = ephy_tab_for_embed (EPHY_EMBED (embed_widget));
 	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return tab;
@@ -1808,15 +2187,25 @@ void
 ephy_window_remove_tab (EphyWindow *window,
 		        EphyTab *tab)
 {
-	GtkWidget *embed;
+	EphyEmbed *embed;
+	gboolean modified;
 
 	g_return_if_fail (EPHY_IS_WINDOW (window));
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
-	embed = GTK_WIDGET (ephy_tab_get_embed (tab));
+	embed = ephy_tab_get_embed (tab);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
+
+	modified = ephy_embed_has_modified_forms (embed);
+	if (ephy_embed_has_modified_forms (embed)
+	    && confirm_close_with_modified_forms (window) == FALSE)
+	{
+		/* don't close the tab */
+		return;
+	}
 
 	ephy_notebook_remove_page (EPHY_NOTEBOOK (window->priv->notebook),
-				  embed);
+				   GTK_WIDGET (embed));
 }
 
 /**
@@ -1855,7 +2244,7 @@ ephy_window_show (GtkWidget *widget)
 	EphyWindow *window = EPHY_WINDOW(widget);
 
 	update_chrome (window);
-	update_layout_toggles(window);
+	update_actions(window);
 
 	if (!window->priv->has_size)
 	{
@@ -1925,8 +2314,9 @@ ephy_window_get_tabs (EphyWindow *window)
 	{
 		EphyTab *tab;
 
-		tab = g_object_get_data (G_OBJECT (w), "EphyTab");
-		g_return_val_if_fail (EPHY_IS_TAB (G_OBJECT (tab)), NULL);
+		g_return_val_if_fail (EPHY_IS_EMBED (w), NULL);
+		tab = ephy_tab_for_embed (EPHY_EMBED (w));
+		g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 		tabs = g_list_prepend (tabs, tab);
 		i++;
@@ -2008,6 +2398,29 @@ ephy_window_find (EphyWindow *window)
 void
 ephy_window_print (EphyWindow *window)
 {
+	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINTING))
+	{
+		return;
+	}
+
+	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) ||
+	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE))
+	{
+		EphyEmbed *embed;
+		EmbedPrintInfo *info;
+
+		info = ephy_print_get_print_info ();
+
+		embed = ephy_window_get_active_embed (window);
+		g_return_if_fail (EPHY_IS_EMBED (embed));
+
+		ephy_embed_print (embed, info);
+
+		ephy_print_info_free (info);
+
+		return;
+	}
+
 	if (window->priv->print_dialog == NULL)
 	{
 		EphyDialog *dialog;
